//!The RAq Approach: tree.h
/*!
Contains supporting classes and functions for the creation of the decomposition tree (Tree)
and the supertree (SuperTree). Basic Tree class is based off of template found on library.thinkquest.org
*/
//(c) 2006 Suzanne Matthews
/*
This file is part of the RaqApproach software package.

    RaqApproach is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RaqApproach is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RaqApproach; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

//basic tree class based off of template found on library.thinkquest.org
#ifndef TREE_H_
#define TREE_H_

#include "common.h"

using namespace std;

typedef list<element> elements;
typedef vector<short> nid;

//! \struct a Node struct
/*! Node is the data structure primarily used for the decomposition step
	\param *left pointer to the left child
	\param *right pointer to the right child
	\param *parent pointer to the parent of current node
	\param id identifier vector of the node
	*/
struct Node
{
   elements data;
   Node *left;
   Node *right;
   Node *parent;
   nid id; //identification of the node
};

//! \struct a SuperNode struct
/*! SuperNode is the data structure primarily used for the super tree construction step
	\param taxa holds taxa. is 0 when node is internal
	\param *left pointer to the left child
	\param *right pointer to the right child
	\param *parent pointer to the parent
	\param internal indicates whether or not is internal node
	\param id identification of the node (-1 when it is not the root node)
	*/
struct SuperNode
{
   int taxa; //!< is 0 if there is no taxa: only occurs when it is an internal node
   SuperNode *left; //!< would be NULL if is a leaf node
   SuperNode *right; //!< would be NULL if is a leaf node
   SuperNode *parent; //!< is never NULL... except when it is the root
   bool internal; //!< indicates whether or not is an internal node
   nid id; //!< identification of the node
};

//! displays the node pointed to by curr
/*! \param curr pointer that points to the the node that should be displayed*/
void display_node(Node * curr);
//! displays the SuperNode pointed to by curr
/*! \param curr pointer that points to the SuperNode that should be displayed*/
void display_SuperNode(SuperNode * curr);

//!\class Tree class
/*! Basic data structure that creates the decomposition tree and guide tree in the
    decomposition step*/
class Tree
{
public:
   //! create empty tree
   /*! creates tree with default root node which has no value.
       set current to main root node.*/
   Tree();

   //! create new tree with passed node as the new main root.
   /*! set current to main root. if the second parameter is 0, the new object simply
       points to the node of the original tree. If the second parameter is 1, a new
       copy of the subtree is created, which the object points to.
       \param Node* indicates root location
       \param int indicates where object points to (0: node of original tree, 1: new subtree)
       */
   Tree(Node*,int);

   //! destructor
   ~Tree();
   //! inserts a new node as child of current
   /*! inserts a new element into tree at desired position.
      \param elements& list<element> that should be inserted into tree
      \param int indicates position that node should be inserted (0: left 1:right)
      */
   void insert(const elements&,int);
   //! deletes a node and its subtree
   /*! recursively removes the node pointed to by Node* and all of its subtress
      \param Node* points to root of tree that is to be deleted*/
   void remove(Node*); //delete node and its subtree
   //! copy constructor
   /*! performs a "deep copy" of tree object
      \sa Tree(), Tree(Node*, int)
      */
   Tree(const Tree &);
   //! value function
   /*! \return list<element> of current Node
      \sa Id()
      */
   elements value() const;
   //! Id function
   /*! \return corresponding id of current Node (vector<short>)
      \sa value()
    */
   nid Id() const; //display level of the current node

   //! navigate the tree
   void left(); //!< go left
   void right(); //!< go right
   void parent(); //!< go the the parent
   void reset(); //!< go to main_root
   //! set placement of the current pointer
   /*! \param Node* points to location that current should be set to*/
   void SetCurrent(Node*);

   //! return subtree (node) pointers
   Node* pointer_left() const; //!< returns pointer to left child
   Node* pointer_right() const; //!< returns pointer to right child
   Node* pointer_parent() const; //!< returns pointer to parent
   Node* pointer_current() const; //!< returns current pointer
   Node * root() const; //!< returns pointer to root of Tree

   //! return values of children and parent without leaving current node
   elements peek_left() const; //!< returns elements of left child
   elements peek_right() const; //!< returns elements of right child
   elements peek_parent() const; //!< returns elements of parent

   //! print the tree or a subtree.
   void DisplayInorder(Node*) const; //!< print an "in-order" traversal
   void DisplayPreorder(Node*) const; //!< print a "pre-order" traversal
   void DisplayPostorder(Node*) const; //!< print a "post-order" traversal

   //! delete all nodes in the tree
   void clear();

   //! check to see if the tree is empty or full
   bool IsEmpty() const;
   bool IsFull() const;

private:
   Node* current; //!< pointer to current Node
   Node* main_root; //!< pointer to the root of the Tree
   Node* CopyTree(Node*,Node*) const; //!< create a new copy of a subtree if passed to the constructor
   bool subtree; //!< does it reference a part of a larger object?
};

//!\class SuperTree class
/*! Basic data structure that creates the subtrees and final supertree in the final steps
    of the algorithm*/
class SuperTree
{
public:

   //! create empty tree
   /*! creates empty tree with default root node which has no value.
      set current to main root node.*/
   SuperTree();
   //!create new tree with passed node as the new main root.
   /*! set current to main root.
      \param SuperNode* node to set as root of tree
      \param int indicates where object should point to (0: node of original tree 1: new copy of the subtree
      \sa SuperTree(SuperNode*, int), SuperTree(const SuperTree &), ~SuperTree()
   */
   SuperTree(SuperNode*,int);
   //! destructor
   /*! \sa SuperTree(), SuperTree(SuperNode*, int), SuperTree(const SuperTree &)*/
   ~SuperTree();
   //! insert new node as child of current.
   /*! insert new taxa into supertree:
      \param int taxa (-1 or 0 if it is an internal node)
      \param int location to insert (0=left 1=right 2 = parent)
      \param nid id of Node to be inserted (-1 if is an internal node)
      \param bool indicates whether or not the node is internal
      \sa insert(SuperTree &, SuperTree &)
      */
   void insert(int, int, nid, bool);
   //! overloaded insert fuction
   /*! inserts a right and left supertree as subtrees
      \param SuperTree & left tree to be inserted
      \param SuperTree & right tree to be inserted
      \sa insert(int, int, nid, bool)
      */
   void insert(SuperTree &, SuperTree &);
   //! delete a node and its subtree
   /*! recursively deletes the node pointed to by SuperNode and all the nodes in its subtree
      \param SuperNode* indicates root of tree to be deleted
      \sa clear()
   */
   void remove(SuperNode*);
   //! value function
   /*! \return taxa at current location*/
   int value() const;
   //! Id function
   /*! \return id at current location (vector<short>)*/
   nid Id() const; //display level of the current node

   //! navigate the tree
   void left(); //!< go to left child
   void right(); //!< go to right child
   void parent(); //!< go to parent
   void reset(); //!< go to main_root
   //! set the location of the current pointer
   /*! \param SuperNode* location that current pointer should be set to*/
   void SetCurrent(SuperNode*);
   //! copy constructor
   /*! allows SuperTree to do a "deep" copy
       \param SuperTree& SuperTree object to be copied
       */
   SuperTree(const SuperTree &);

   //! return subtree (node) pointers
   SuperNode* pointer_left() const; //!< returns pointer to left child
   SuperNode* pointer_right() const; //!< returns pointer to right child
   SuperNode* pointer_parent() const; //!< returns pointer to parent
   SuperNode* pointer_current() const; //!< returns current pointer
   SuperNode* root() const; //!< returns pointer to main_root

   //! return values of children and parent without leaving current node
   int peek_left() const; //!< return taxa of left child
   int peek_right() const; //!< return taxa of right child
   int peek_parent() const; //!< return taxa of parent

   //! print the tree or a subtree.
   void DisplayInorder(SuperNode*) const; //!< do an "in-order" traversal
   void DisplayPreorder(SuperNode*) const; //!< do a "pre-order" traversal
   void DisplayPostorder(SuperNode*) const; //!< do a "post-order" traversal
   //! displays newick format of tree
   /*! recrusively outputs the tree in Newick format
      \param list<string>& contains tree so far in Newick format
      \param SuperNode* pointer to root of tree we want to output in Newick format
    */
   void Newick(list<string> &, SuperNode*);

   //!delete all nodes in the tree
   void clear();

   bool IsEmpty() const; //!< checks to see if a tree is empty
   bool IsFull() const; //!< checks to see if the tree is full
   bool IsInternal() const; //!< checks to see the current node is an internal node

private:
   SuperNode* current; //!< pointer to current node
   SuperNode* main_root; //!< pointer to root node
   SuperNode* CopyTree(SuperNode*,SuperNode*) const; //!< create a new copy of a subtree if passed to the constructor
   SuperNode* CopyTree(SuperNode *, SuperNode *, int) const; //!< CopyTree created for personal devices
   bool subtree; //!< does it reference a part of a larger object?
};

//! Default constructor simply sets the main_root and current data members to NULL
/*! Subtree value is false*/
Tree::Tree()
{
   //create a root node with no value
   main_root = NULL;
   current = NULL;
   subtree = false;
}

//! Default constructor simply sets the main_root and current data members to NULL
/*! Subtree value is false*/
SuperTree::SuperTree()
{
   //create a root node with no value
   main_root = NULL;
   current = NULL;
   subtree = false;
}

//!copy constructor
/*! Accepts a pointer to a node, and creates a new tree object with the node that is
  passed acting as the new tree's main root.current is then set to the main root.
  The second parameter specifies whether the new subtree object points directly
  to the original tree's nodes (the root and its decedents), or creates a copy of
  the subtree and is thus a new tree. The subtree variable specifies if the subtree
  points directly to the original tree's nodes. important in the class destructor.
  \param root points to the root of tree
  \param op indicates where object should point to
  \sa Tree(), Tree(Tree&), ~Tree()
*/
Tree::Tree(Node* root, int op)
{
   if(op == 0)
   {
      main_root = root;
      current = root;
      subtree = true;
   }
   else
   {
      main_root = CopyTree(root,NULL);
      current = main_root;
      subtree = false;
   }
}

//!copy constructor
/*! Accepts a pointer to a node, and creates a new tree object with the node that is
  passed acting as the new tree's main root.current is then set to the main root.
  The second parameter specifies whether the new subtree object points directly
  to the original tree's nodes (the root and its decedents), or creates a copy of
  the subtree and is thus a new tree. The subtree variable specifies if the subtree
  points directly to the original tree's nodes. important in the class destructor.
  \param root points to the root of tree
  \param op indicates where object should point to
  \sa SuperTree(), SuperTree(Tree&), ~SuperTree()
*/
SuperTree::SuperTree(SuperNode* root, int op)
{
   if(op == 0)
   {
      main_root = root;
      current = root;
      subtree = true;
   }
   else
   {
      main_root = CopyTree(root,NULL);
      current = main_root;
      subtree = false;
   }
}

Node* Tree::root() const {
	return main_root;
}

Tree::Tree(const Tree & tr) {
	main_root = CopyTree(tr.root(), NULL);
}

SuperNode* SuperTree::root() const {
	return main_root;
}

SuperTree::SuperTree(const SuperTree & tr) {
	main_root = CopyTree(tr.root(), NULL);
}

//! The CopyTree() function creates a copy of subtree root and returns a pointer location of the new copy's root node.
/*! The second parameter is a pointer to the parent of the subtree being passed.
    Since parent of the main root is always NULL, we pass NULL as the second parameter
    in the class constructor
    \param root pointer to root of tree
    \param parent pointer to parent of Node
    \return pointer location of new root node
 */
Node* Tree::CopyTree(Node *root, Node *parent) const
{
   if(root == NULL) //!< base case - if the node doesn't exist, return NULL.
      return NULL;
   Node* tmp = new Node; //!< make a new location in memory
   tmp->data = root->data; //!< make a copy of the node's data
   tmp->parent = parent; //!< set the new node's parent
   tmp->left = CopyTree(root->left,tmp); //!< copy the left subtree of the current node. pass the current node as the subtree's parent
   tmp->right = CopyTree(root->right,tmp); //!< do the same with the right subtree
   tmp->id = root->id;
   return tmp; //!< return a pointer to the newly created node.
}

//! The CopyTree() function creates a copy of subtree root and returns a pointer location of the new copy's root node.
/*! The second parameter is a pointer to the parent of the subtree being passed.
    Since parent of the main root is always NULL, we pass NULL as the second parameter
    in the class constructor
    \param root pointer to root of SuperTree
    \param parent pointer to parent of SuperNode
    \return pointer location of new root node
 */
SuperNode* SuperTree::CopyTree(SuperNode *root, SuperNode *parent) const
{
   if(root == NULL) //!< base case - if the node doesn't exist, return NULL.
      return NULL;
   SuperNode* temp = new SuperNode; //!< make a new location in memory
   temp->taxa = root->taxa; //!< make a copy of the node's data
   temp->parent = parent; //!< set the new node's parent
   temp->left = CopyTree(root->left,temp); //!< copy the left subtree of the current node. pass the current node as the subtree's parent
   temp->right = CopyTree(root->right,temp); //!< do the same with the right subtree
   temp->id = root->id;
   temp->internal = root->internal;
   return temp; //!< return a pointer to the newly created node.
}

//! The CopyTree() function for one's own devices
/*! Does same thing as the one above, accept is created for own devices
    sets the ids of all the internal nodes to -1
    \param root pointer to root of SuperTree
    \param parent pointer to parent of SuperNode
    \param dummy just a dummy value to indicate a new function (boo! bad coding practice!)
    \return pointer location of new root node
 */
SuperNode* SuperTree::CopyTree(SuperNode *root, SuperNode *parent, int dummy) const
{
   if(root == NULL) //!< base case - if the node doesn't exist, return NULL.
      return NULL;
   SuperNode* temp = new SuperNode; //!< make a new location in memory
   temp->taxa = root->taxa; //!< make a copy of the node's data
   temp->parent = parent; //!< set the new node's parent
   temp->left = CopyTree(root->left,temp); //!< copy the left subtree of the current node. pass the current node as the subtree's parent
   temp->right = CopyTree(root->right,temp); //!< do the same with the right subtree
   temp->id.push_back(-1); //!< what makes this function different from the other one!
   temp->internal = root->internal;
   return temp; //!< return a pointer to the newly created node.
}

//! Class destructor
/*! calls the clear function to recursively remove subtrees*/
Tree::~Tree()
{
   if(!subtree)
      clear(); //!< delete all nodes
}
//! Class destructor
/*! calls the clear function to recursively remove subtrees*/
SuperTree::~SuperTree()
{
   if(!subtree)
      clear(); //!< delete all nodes
}

//! The insert() function creates a new node as a child of current.
/*! Creates a node and inserts it into the tree
  \param item list<element> to be inserted into tree
  \param pos position to enter node into tree (0:left 1:right)
  */
void Tree::insert(const elements &item,int pos) //!<insert as child of current 0=left 1=right. if item already exists, replace it
{
   assert(!IsFull());
   //! if the tree has no nodes, make a root node, disregard pos.
   if(main_root == NULL)
   {
      main_root = new Node;
      main_root->data = item;
      main_root->left = NULL;
      main_root->right = NULL;
      main_root->parent = NULL;
      main_root->id.push_back(0);
      current = main_root;
      return; //!< node created, exit the function
   }

   if(pos == 0) //!< if new node is a left child of current
   {
      if(current->left != NULL) //!< if child already exists, replace value
         (current->left)->data = item;
      else
      {
         current->left = new Node;
         current->left->data = item;
         current->left->left = NULL;
         current->left->right = NULL;
         current->left->id = current->id; //!< if is left child, copy the id of the parent
         current->left->id.push_back(1); //!< push_back the value one (we're creating a new level)
         current->left->parent = current;
      }
   }
   else //!< else, new node is a right child of current
   {
      if(current->right != NULL) //!< if child already exists, replace value
         (current->right)->data = item;
      else
      {
         current->right = new Node;
         current->right->data = item;
         current->right->left = NULL;
         current->right->right = NULL;
         current->right->id = current->id; //!< if it is a right child, copy the id of the parent
         short temp = current->right->id.back(); //!< increment the last element by one (we're on the same level)
         temp++;
         current->right->id.pop_back();
         current->right->id.push_back(temp);
         current->right->parent = current;
      }
   }
}

//! The insert() function creates a new node as a child of current.
/*! Creates a node and inserts it into the tree
  \param taxa taxa to be inserted into tree
  \param pos position to enter node into tree (0:left 1:right 2:parent)
  \param id_ id of node to be inserted into tree
  \param is_internal indicates if node is an internal node or not
  */
void SuperTree::insert(int tax, int pos, nid id_, bool is_internal) //insert as child of current 0=left 1=right. if item already exists, replace it
{
   assert(!IsFull());
   //! if the tree has no nodes, make a root node, disregard pos.
   if(main_root == NULL)
   {
      main_root = new SuperNode;
      main_root->taxa = 0;
      main_root->left = NULL;
      main_root->right = NULL;
      main_root->parent = NULL;
      main_root->id = id_;
      main_root->internal = 1;
      current = main_root;
      return; //!< node created, exit the function
   }

   if(pos == 0) //!< if new node is a left child of current
   {
      if(current->left != NULL) {  //!< if child already exists, replace value
         (current->left)->taxa = tax;
        cout << "we shouldn't get in here!!!" << endl;
	  }
      else
      {
         current->left = new SuperNode;
         current->left->taxa = tax;
         current->left->left = NULL;
         current->left->right = NULL;
         current->left->id.push_back(-1); //!< if it is a left child, copy the id of the parent
         current->left->parent = current;
         current->left->internal = is_internal;
      }
   }
   else if(pos == 1) //!< else if new node is a right child of current
   {
      if(current->right != NULL) { //!< if child already exists, replace value
         (current->right)->taxa = tax;
         cout << "we shouldn't get in here, either!" << endl;
	  }
      else
      {
         current->right = new SuperNode;
         current->right->taxa = tax;
         current->right->left = NULL;
         current->right->right = NULL;
         current->right->id.push_back(-1); //!< if it is a right child, copy the id of the parent
         current->right->parent = current;
         current->right->internal = is_internal;
         }
   }
   else {
	   SuperNode * temp_left = current->parent->left;
	   SuperNode * temp_right = current->parent->right;
	   current->parent = new SuperNode;
	   current->parent->taxa = 0;
	   current->parent->internal = 1;
	   current->parent->left = temp_left;
	   current->parent->right = temp_right;
	   current->parent->internal = is_internal;
	   current->parent->id = id_;
	   current->parent->id.pop_back();
	   current = current;
   }
}

//! insert a left and right subtree into a tree
/*! useful when we want to a left and right subtree into our current tree
    \param left left tree to be inserted
    \param right right tree to be inserted
    \sa insert(int, int, nid, bool)
 */
void SuperTree::insert(SuperTree & left, SuperTree & right) //!< insert left and right subtrees
{
   vector<short> id_ = left.root()->id;
   assert(!IsFull());
   //!< if the tree has no nodes, make a root node, disregard pos.
   if(main_root == NULL)
   {
      main_root = new SuperNode;
      main_root->taxa = 0;
      main_root->left = CopyTree(left.root(), main_root, 0);
      main_root->right = CopyTree(right.root(), main_root, 0);
      main_root->parent = NULL;
      main_root->id = id_;
      main_root->internal = 1;
      main_root->left->id.clear();
      main_root->left->id.push_back(-1);
      main_root->right->id.clear();
      main_root->right->id.push_back(-1);
      current = main_root;
      return; //!< node created, exit the function
   }
}

//! value function
/*! \return list<element> containing current taxa sets*/
elements Tree::value() const
{
   return current->data;
}

//! value function
/*! \return taxa pointed to by current pointer*/
int SuperTree::value() const
{
   return current->taxa;
}

//! Id function
/*! \return id pointed to by current*/
nid Tree::Id() const
{
	return current->id;
}

//! Id function
/*! \return id pointed to by current*/
nid SuperTree::Id() const
{
	return current->id;
}

//! The next five functions are used to navigate the tree (self explanatory).

void Tree::left()
{
   current = current->left;
}

void Tree::right()
{
   current = current->right;
}

void Tree::parent()
{
   current = current->parent;
}

void Tree::reset()
{
   current = main_root;
}

void Tree::SetCurrent(Node* root)
{
   current = root;
}

//! The next five functions are used to navigate the SuperTree (self-explanatory).
void SuperTree::left()
{
   current = current->left;
}

void SuperTree::right()
{
   current = current->right;
}

void SuperTree::parent()
{
   current = current->parent;
}

void SuperTree::reset()
{
   current = main_root;
}

void SuperTree::SetCurrent(SuperNode* root)
{
   current = root;
}

//! The next four functions return pointers to various nodes in the tree (self-explanatory)

Node* Tree::pointer_left() const
{
   return current->left;
}

Node* Tree::pointer_right() const
{
   return current->right;
}

Node* Tree::pointer_parent() const
{
   return current->parent;
}

Node* Tree::pointer_current() const
{
   return current;
}

//! The next four functions return pointers to various nodes in the SuperTree (self-explanatory)
SuperNode* SuperTree::pointer_left() const
{
   return current->left;
}

SuperNode* SuperTree::pointer_right() const
{
   return current->right;
}

SuperNode* SuperTree::pointer_parent() const
{
   return current->parent;
}

SuperNode* SuperTree::pointer_current() const
{
   return current;
}


//! The next three functions return the values of a node's two children and parent
/*! advantage: we don't have to leave the node! (self-explanatory)*/
elements Tree::peek_left() const
{
   assert(current->left != NULL);
   return current->left->data;
}

elements Tree::peek_right() const
{
   assert(current->right != NULL);
   return current->right->data;
}

elements Tree::peek_parent() const
{
   assert(current->parent != NULL);
   return current->parent->data;
}

//! The next three functions return the values of a node's two children and parent
/*! advantage: we don't have to leave the node! (self-explanatory)*/
int SuperTree::peek_left() const
{
   assert(current->left != NULL);
   return current->left->taxa;
}

int SuperTree::peek_right() const
{
   assert(current->right != NULL);
   return current->right->taxa;
}

int SuperTree::peek_parent() const
{
   assert(current->parent != NULL);
   return current->parent->taxa;
}

//! The display functions for Tree

void Tree::DisplayInorder(Node* root) const
{
   if (root == NULL)
      return;

   DisplayInorder(root->left);
   display_node(root);
   DisplayInorder(root->right);
}

void Tree::DisplayPreorder(Node* root) const
{
   if (root == NULL)
      return;

   display_node(root);
   DisplayPreorder(root->left);
   DisplayPreorder(root->right);
}

void Tree::DisplayPostorder(Node* root) const
{
   if (root == NULL)
      return;

   DisplayPostorder(root->left);
   DisplayPostorder(root->right);
   display_node(root);
}

//! The display functions for SuperTree
void SuperTree::DisplayInorder(SuperNode* root) const
{
   if (root == NULL)
      return;

   DisplayInorder(root->left);
   display_SuperNode(root);
   DisplayInorder(root->right);
}

void SuperTree::DisplayPreorder(SuperNode* root) const
{
   if (root == NULL)
      return;

   display_SuperNode(root);
   DisplayPreorder(root->left);
   DisplayPreorder(root->right);
}

void SuperTree::DisplayPostorder(SuperNode* root) const
{
   if (root == NULL )
   	  return;

   DisplayPostorder(root->left);
   DisplayPostorder(root->right);
   display_SuperNode(root);
}

//! Display function (Newick)
void SuperTree::Newick(list<string> & newick_, SuperNode * curr ) {

    if ( curr == NULL )
    	return;

    if ( curr->internal == 0 ) {
		int temp = curr->taxa;
		stringstream ss1;
		string temp_taxa;
		ss1 << temp;
		ss1 >> temp_taxa;
		newick_.push_back(temp_taxa);
	}
	else {
		newick_.push_back("(");
		Newick(newick_, curr->left);
		newick_.push_back(",");
		Newick(newick_, curr->right);
		newick_.push_back(")");
	}


}

//!The clear() function deletes all nodes in the list.
void Tree::clear()
{
   remove(main_root); //!< use the remove function on the main root
   main_root = NULL; //!< since there are no more items, set main_root to NULL
   current = NULL;
}

void Tree::remove(Node* root)
{
   if(root == NULL) //!< base case - if the root doesn't exist, do nothing
      return;
   remove(root->left); //!< perform the remove operation on the nodes left subtree first
   remove(root->right); //!< perform the remove operation on the nodes right subtree first
   if(root->parent == NULL) //!< if the main root is being deleted, main_root must be set to NULL
      main_root = NULL;
   else
   {
      if(root->parent->left == root) //!< make sure the parent of the subtree's root points to NULL, since the node no longer exists
         root->parent->left = NULL;
      else
         root->parent->right = NULL;
   }
   current = root->parent; //!< set current to the parent of the subtree removed.
   delete root;
}

//!The clear() function deletes all SuperNodes in the list.
void SuperTree::clear()
{
   remove(main_root); //!< use the remove function on the main root
   main_root = NULL; //!< since there are no more items, set main_root to NULL
   current = NULL;
}

void SuperTree::remove(SuperNode* root)
{
   if(root == NULL) //!< base case - if the root doesn't exist, do nothing
      return;
   remove(root->left); //!< perform the remove operation on the nodes left subtree first
   remove(root->right); //!< perform the remove operation on the nodes right subtree first
   if(root->parent == NULL) //!< if the main root is being deleted, main_root must be set to NULL
      main_root = NULL;
   else
   {
      if(root->parent->left == root) //!< make sure the parent of the subtree's root points to NULL, since the node no longer exists
         root->parent->left = NULL;
      else
         root->parent->right = NULL;
   }
   current = root->parent; //!< set current to the parent of the subtree removed.
   delete root;
}

//! The IsEmpty() function works by evaluating main_root..
bool Tree::IsEmpty() const
{
   return (main_root == NULL); //!< If there aren't any nodes in the tree, main_root points to NULL
}

//! The IsEmpty() function works by evaluating main_root..
bool SuperTree::IsEmpty() const
{
   return (main_root == NULL); //!< If there aren't any nodes in the tree, main_root points to NULL
}

//! checks to see if tree is full
bool Tree::IsFull() const
{
   Node *tmp = new Node;
   if(tmp == NULL)
      return true;
   else
   {
      delete tmp;
      return false;
   }
}

//! displays node pointed to by curr
void display_node(Node * curr) {
	cout << "outputting node... " << endl;
	cout << "tree id: |";
	nid temp = curr->id;
	for (unsigned int i = 0; i < temp.size(); ++i)
		cout << temp[i] << " | ";
	cout << endl;
	elements temp_data = curr->data;
	cout << "clustered elements: " << endl;
	elements::iterator te_itr = temp_data.begin();
	while ( te_itr != temp_data.end() ) {
		cout << (*te_itr).get_taxa() << endl;
		++te_itr;
	}
}

//! checks to see if SuperTree is full
bool SuperTree::IsFull() const
{
   SuperNode *tmp = new SuperNode;
   if(tmp == NULL)
      return true;
   else
   {
      delete tmp;
      return false;
   }
}

//! displays node pointed to by curr
void display_SuperNode(SuperNode * curr) {
	cout << "outputting super node... " << endl;
	cout << "tree id: |";
	nid temp = curr->id;
	for (unsigned int i = 0; i < temp.size(); ++i)
		cout << temp[i] << " | ";
	cout << endl;
	if ( curr->internal == 1 ) {
		cout << "internal node" << endl;
	}
	else {
		cout << "taxa " << curr->taxa << endl;
	}
	cout << endl;
}

//!this is for the merger:
/*! display all the subtrees that will be merged, in the order of importance
   \param curr points to root
   */
void show_leaves(Node * curr) {
	if ( curr->left != NULL ) {
		show_leaves(curr->left);
		if ( curr->right != NULL )
			show_leaves(curr->right);

	}
	else { //!< if curr->left == NULL
		display_node(curr);

		if ( curr->right != NULL )
			show_leaves(curr->right); //!< if they are both null, we still output this
	}
}

//!this is for the merger:
/*! add all subtrees to be merged into leaves, in the order of importance
   \param curr points to root
   */
void find_leaves(list<Node *> & leaves, Node * curr) {
	if ( curr->left != NULL ) {
		find_leaves(leaves, curr->left);
		if ( curr->right != NULL )
			find_leaves(leaves, curr->right);

	}
	else { //!< if curr->left == NULL
		leaves.push_back(curr);
		if ( curr->right != NULL )
			find_leaves(leaves, curr->right); //!< if they are both null, we still output this
	}
}

//! merge the trees pointed by the list of supertrees
/*! recursively merges a list of trees together into one big supertree
   \param many_trees list of trees to be merged into one large tree
   */
void merge_trees(list<SuperTree> & many_trees) {
	if ( many_trees.size() == 1 ) //!< if there one tree in list
		return; //!< return (base case)
	SuperTree left = many_trees.front(); //!< grab first tree
	many_trees.pop_front();
	SuperTree right = many_trees.front(); //!< grab second tree
	many_trees.pop_front();

	//! create a simple tree structure out of them
	SuperTree new_tree;
	new_tree.insert(left, right);
	if ( DISPLAY ) {
		cout << "displaying the NEWLY CREATED TREE." << endl; //<! output tree (optional)
		new_tree.DisplayInorder(new_tree.root());
	}
	many_trees.push_front(new_tree); //!< add newly created tree to front of list
	if ( many_trees.size() > 1 ) //!< if we still have more than one tree in list
		merge_trees(many_trees); //!< merge the trees together
	else
		return; //!< else return
}

//! count the leaf nodes in the tree
int LeafCount(Node* root)
{
   if(root == NULL) //!< base case - if the node doesn't exist, return 0 (don't count it)
      return 0;
   if((root->left == NULL) && (root->right == NULL)) //!< if the node has no children return 1 (it is a leaf)
      return 1;
   return LeafCount(root->left) + LeafCount(root->right); //!< add the leaf nodes in the left and right subtrees
}

//! count the nodes in the tree
int NodeCount(Node* root)
{
   if(root == NULL) //!< base case - if the return 0 if node doesn't exist (don't count it)
      return 0;
   else
      return 1 + NodeCount(root->left) + NodeCount(root->right); //!< return 1 for the current node, and add the amount of nodes in the left and right subtree
}

#endif
